package com.iformall.service.wechat;

import com.iformall.config.WechatRedisProperies;
import com.iformall.config.WechatOpenProperties;
import com.iformall.domain.po.WxAuthorizerInfo;
import com.iformall.enums.EnumAppType;
import com.iformall.mapper.WxAuthorizerInfoMapper;
import com.iformall.mapper.WxComponentVerifyTicketMapper;
import com.iformall.service.wechat.handler.*;
import me.chanjar.weixin.common.api.WxConsts;
import me.chanjar.weixin.mp.constant.WxMpEventConstants;
import me.chanjar.weixin.open.api.impl.WxOpenMessageRouter;
import me.chanjar.weixin.open.api.impl.WxOpenServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.stereotype.Service;
import redis.clients.jedis.JedisPool;

import javax.annotation.PostConstruct;

/**
 * Stormeye WU
 */
@Service
@EnableConfigurationProperties({WechatOpenProperties.class, WechatRedisProperies.class})
public class FmOpenService extends WxOpenServiceImpl {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private WechatOpenProperties wechatProperties;
    @Autowired
    private WechatRedisProperies redisProperies;

    @Autowired
    private WxComponentVerifyTicketMapper componentVerifyTicketMapper;

    @Autowired
    private WxAuthorizerInfoMapper authorizerInfoMapper;

    @Autowired
    protected NullHandler nullHandler;
    @Autowired
    protected KfSessionHandler kfSessionHandler;
    @Autowired
    protected StoreCheckNotifyHandler storeCheckNotifyHandler;
    @Autowired
    protected LocationHandler locationHandler;
    @Autowired
    protected MenuHandler menuHandler;
    @Autowired
    protected MsgHandler msgHandler;
    @Autowired
    protected UnsubscribeHandler unsubscribeHandler;
    @Autowired
    protected SubscribeHandler subscribeHandler;

    @Autowired
    protected WeappAuditHandler weappAuditHandler;

    @Autowired
    protected CardCheckHandler cardCheckHandler;  // 2.1
    @Autowired
    protected CardUserGetCardHandler cardUserGetCardHandler; // 2.2
    @Autowired
    protected CardUserGiftingCardHandler cardUserGiftingCardHandler; // 2.3
    @Autowired
    protected CardUserDelCardHandler cardUserDelCardHandler; // 2.4
    @Autowired
    protected CardUserConsumeCardHandler cardUserConsumeCardHandler; // 2.5
    @Autowired
    protected CardUserPayHandler cardUserPayHandler;  // 2.6
    @Autowired
    protected CardUserViewCardHandler cardUserViewCardHandler; // 2.7
    @Autowired
    protected CardUserEnterSessionHandler cardUserEnterSessionHandler; // 2.8
    @Autowired
    protected CardUpdateMemberCardHandler cardUpdateMemberCardHandler; // 2.9
    @Autowired
    protected CardSkuRemindHandler cardSkuRemindHandler; // 2.10
    @Autowired
    protected CardPayOrderHandler cardPayOrderHandler;   // 2.11
    @Autowired
    protected CardSubmitMemCardHandler cardSubmitMemCardHandler; // 2.12


    private static JedisPool pool;
    private WxOpenMessageRouter wxOpenMessageRouter;

    @PostConstruct
    public void init() {
        FmOpenInRedisDBConfigStorage configStorage = new FmOpenInRedisDBConfigStorage(getJedisPool(), componentVerifyTicketMapper, authorizerInfoMapper);
        configStorage.setComponentAppId(wechatProperties.getComponentAppId());
        configStorage.setComponentAppSecret(wechatProperties.getComponentSecret());
        configStorage.setComponentToken(wechatProperties.getComponentToken());
        configStorage.setComponentAesKey(wechatProperties.getComponentAesKey());
        setWxOpenConfigStorage(configStorage);
        wxOpenMessageRouter = new WxOpenMessageRouter(this);
        wxOpenMessageRouter.rule().handler((wxMpXmlMessage, map, wxMpService, wxSessionManager) -> {
            String appId = wxMpService.getWxMpConfigStorage().getAppId();
            logger.info("\n接收到 {} 请求消息，内容：{}", appId, wxMpXmlMessage);
            return null;
        }).next();

        // 接收客服会话管理事件
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxMpEventConstants.CustomerService.KF_CREATE_SESSION)
                .handler(this.kfSessionHandler).end();
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxMpEventConstants.CustomerService.KF_CLOSE_SESSION)
                .handler(this.kfSessionHandler).end();
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxMpEventConstants.CustomerService.KF_SWITCH_SESSION)
                .handler(this.kfSessionHandler).end();

        // 门店审核事件
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxMpEventConstants.POI_CHECK_NOTIFY)
                .handler(this.storeCheckNotifyHandler).end();

        // 自定义菜单事件
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.MenuButtonType.CLICK)
                .handler(this.menuHandler).end();

        // 点击菜单连接事件
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.MenuButtonType.VIEW)
                .handler(this.nullHandler).end();

        // 关注事件
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.SUBSCRIBE)
                .handler(this.subscribeHandler).end();

        // 取消关注事件
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.UNSUBSCRIBE)
                .handler(this.unsubscribeHandler).end();

        // 上报地理位置事件
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.LOCATION)
                .handler(this.locationHandler).end();

        // 接收地理位置消息
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.LOCATION)
                .handler(this.locationHandler).end();

        // 扫码事件
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.SCAN)
                .handler(this.nullHandler).end();

        // 小程序 审核通过
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.WEAPP_AUDIT_SUCCESS)
                .handler(this.weappAuditHandler).end();
        // 小程序 审核失败
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.WEAPP_AUDIT_FAIL)
                .handler(this.weappAuditHandler).end();

        // 卡券事件推送
        //     审核事件推送
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_PASS_CHECK)
                .handler(this.cardCheckHandler).end();
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_NOT_PASS_CHECK)
                .handler(this.cardCheckHandler).end();
        //     领取事件推送
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_USER_GET_CARD)
                .handler(this.cardUserGetCardHandler).end();
        //     转赠事件推送
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_USER_GIFTING_CARD)
                .handler(this.cardUserGiftingCardHandler).end();
        //     删除事件推送
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_USER_DEL_CARD)
                .handler(this.cardUserDelCardHandler).end();
        //     核销事件推送
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_USER_CONSUME_CARD)
                .handler(this.cardUserConsumeCardHandler).end();
        //     买单事件推送
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_USER_PAY_FROM_PAY_CELL)
                .handler(this.cardUserPayHandler).end();
        //     进入会员卡事件推送
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_USER_VIEW_CARD)
                .handler(this.cardUserViewCardHandler).end();
        //     从卡券进入公众号会话事件推送
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_USER_ENTER_SESSION_FROM_CARD)
                .handler(this.cardUserEnterSessionHandler).end();
        //     会员卡内容更新事件
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_UPDATE_MEMBER_CARD)
                .handler(this.cardUpdateMemberCardHandler).end();
        //     库存报警事件
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_SKU_REMIND)
                .handler(this.cardSkuRemindHandler).end();
        //     券点流水详情事件
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_PAY_ORDER)
                .handler(this.cardPayOrderHandler).end();
        //     会员卡激活事件推送
        wxOpenMessageRouter.rule().async(false)
                .msgType(WxConsts.XmlMsgType.EVENT)
                .event(WxConsts.EventType.CARD_PAY_ORDER)
                .handler(this.cardSubmitMemCardHandler).end();

        // 默认
        wxOpenMessageRouter.rule().async(false).handler(this.msgHandler).end();
    }

    public WxOpenMessageRouter getWxOpenMessageRouter() {
        return wxOpenMessageRouter;
    }

    private JedisPool getJedisPool() {
        if (pool == null) {
            synchronized (FmOpenService.class) {
                if (pool == null) {
                    pool = new JedisPool(redisProperies, redisProperies.getHost(),
                            redisProperies.getPort(), redisProperies.getConnectionTimeout(),
                            redisProperies.getSoTimeout(), redisProperies.getPassword(),
                            redisProperies.getDatabase(), redisProperies.getClientName(),
                            redisProperies.isSsl(), redisProperies.getSslSocketFactory(),
                            redisProperies.getSslParameters(), redisProperies.getHostnameVerifier());
                }
            }
        }
        return pool;
    }
}
