package com.iformall.schedule;

import com.iformall.domain.po.WxPayAccount;
import com.iformall.domain.vo.WxPayOrderVo;
import com.iformall.mapper.WxPayAccountMapper;
import com.iformall.mapper.WxPayOrderMapper;
import com.iformall.service.WxAppinfoService;
import com.iformall.service.wechat.FmOpenService;
import me.chanjar.weixin.open.api.WxOpenMaService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class UserUnionIdSchedule {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    private final int TIME_OUT_VALUE = 4 * 60 * 1000; //4分钟

    @Autowired
    WxPayOrderMapper payOrderMapper;

    @Autowired
    private FmOpenService openService;

    @Autowired
    private WxPayAccountMapper payAccountMapper;

    @Autowired
    WxAppinfoService wxAppinfoService;

    @Scheduled(cron = "0 */4 * * * *?") // 每4分钟检查一次
    //@Scheduled(cron = "*/10 * * * * ?") // 测试10秒中一次
    public void payOrderGetUnionIdSchedule() {
        Map dateMap = new HashMap();
        Date curDate = new Date();
        Date expireDate = new Date(curDate.getTime() - TIME_OUT_VALUE );

        // 券相关订单
        dateMap.put("startDate", expireDate);
        List<WxPayOrderVo> paidList = payOrderMapper.findListOfPaidOrderByDate(dateMap);
        paidList.parallelStream().forEach( payOrderVo -> {
            getUnionId(payOrderVo);
        });
    }

    /**
     * 根据payOrder信息获取UnionId
     * @param payOrderVo
     */
    public void getUnionId(WxPayOrderVo payOrderVo) {
        if(StringUtils.isBlank(payOrderVo.getOpenId())) {
            return;
        }
        try {
            WxOpenMaService openMaService = openService.getWxOpenComponentService().getWxMaServiceByAppid(payOrderVo.getAppId());
            if(StringUtils.isNotBlank(payOrderVo.getTransactionId())) {
                String openRet = openMaService.getPaidUnionId(payOrderVo.getOpenId(), payOrderVo.getTransactionId(), null, null);
                logger.info(openRet);
            } else {
                WxPayAccount payAccount = payAccountMapper.selectById(payOrderVo.getPayId());
                if(payAccount != null) {
                    String openRet = openMaService.getPaidUnionId(payOrderVo.getOpenId(), null, payAccount.getMchId(), payOrderVo.getPayOrderNo());
                    logger.info(openRet);
                }
            }
        }catch (Exception e) {
            logger.error(e.getMessage());
        }

    }
}